import java.util.regex.*;
import mpl.MPLSet;
import mpl.MPLShutdown;
import mpl.MPLSimpleShutdown;

// this aspect includes policies: 
// - allowed_trusted_reading_and_sending
// - deny_reading_and_sending
public aspect allowed_trusted_reading_and_sending {
   private Pattern trustedAddress = Pattern.compile(
        "^127\\.0\\.0\\..*");
   private boolean filesRead = false;
   private MPLSet<OutputStream> streamSet =
       new MPLSet<OutputStream>();

   // shutdown sequence
   private MPLShutdown mplExit =
      new MPLSimpleShutdown();

// START RULE NO. 1
   pointcut rule1(DatagramSocket mplTarget, DatagramPacket p):
      target(mplTarget) &&
      args(p) &&
      call(void send(DatagramPacket));

   before(DatagramSocket mplTarget, DatagramPacket p):
      rule1(mplTarget, p)
   {
      try {
      if (!( !filesRead ))
      { mplExit.shutdown(); }
      } catch (Exception e) {
         System.err.println(
           "Error: Policy caused an exception" +
           " - execution halted");
         mplExit.shutdown();
      }
   }

// START RULE NO. 2
   pointcut rule2(FileInputStream mplTarget):
      target(mplTarget) &&
      call(int read(..));

   after(FileInputStream mplTarget)
         returning(int mplReturn):
      rule2(mplTarget)
   {
   try {
      filesRead = true;
   } catch (Exception e) {
      System.err.println(
        "Error: Policy caused an exception" +
        " - execution halted");
      mplExit.shutdown();
   }
   }

// START RULE NO. 3
   pointcut rule3(OutputStream mplTarget):
      target(mplTarget) &&
      call(void write(..));

   before(OutputStream mplTarget):
      rule3(mplTarget)
   {
      try {
      if (!( !(filesRead&&
        streamSet.contains(mplTarget)) ))
      { mplExit.shutdown(); }
      } catch (Exception e) {
         System.err.println(
           "Error: Policy caused an exception" +
           " - execution halted");
         mplExit.shutdown();
      }
   }

// START RULE NO. 4
   pointcut rule4():
      call(ServerSocket.new(..));

   after() returning(ServerSocket mplReturn):
      rule4()
   {
   try {
      streamSet = streamSet.union(mplReturn.
        getOutputStream());
   } catch (Exception e) {
      System.err.println(
        "Error: Policy caused an exception" +
        " - execution halted");
      mplExit.shutdown();
   }
   }

// START RULE NO. 5
   pointcut rule5():
      call(Socket.new(..));

   after() returning(Socket mplReturn):
      rule5()
   {
   try {
      if ( (trustedAddress.matcher(mplTarget.
        getInetAddress().getHostAddress()).matches()) )
      {
         streamSet = streamSet.subtract(mplTarget.
        getOutputStream());
      }
      streamSet = streamSet.union(mplReturn.
        getOutputStream());
   } catch (Exception e) {
      System.err.println(
        "Error: Policy caused an exception" +
        " - execution halted");
      mplExit.shutdown();
   }
   }
}