import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.ArrayList;
import mpl.MPLSet;
import mpl.MPLShutdown;
import mpl.MPLSimpleShutdown;

// this aspect includes policies: 
// - message_policy
public aspect message_policy {
   private Integer[] allowedMsgCount = {5, 6, 7};
   private int allowedMsgCharCount = 250;
   private Integer[] msgCounters = {0, 0, 0};
   private int msgCharCounter = 0;
   private int lowerCharLimit = 10;
   private int upperCharLimit = 30;
   private int len;
   private MPLSet<Socket> sockets =
       new MPLSet<Socket>();
   private MPLSet<OutputStream> oStreams =
       new MPLSet<OutputStream>();
   private MPLSet<ObjectOutputStream> ooStreams =
       new MPLSet<ObjectOutputStream>();

   // shutdown sequence
   private MPLShutdown mplExit =
      new MPLSimpleShutdown();

// START RULE NO. 1
   pointcut rule1(OutputStream os):
      args(os) &&
      call(ObjectOutputStream.new(OutputStream));

   before(OutputStream os):
      rule1(os)
   {
      try {
      if (!( oStreams.contains(os) ))
      { mplExit.shutdown(); }
      } catch (Exception e) {
         System.err.println(
           "Error: Policy caused an exception" +
           " - execution halted");
         mplExit.shutdown();
      }
   }

   after(OutputStream os)
         returning(ObjectOutputStream mplReturn):
      rule1(os)
   {
   try {
      ooStreams = ooStreams.union(mplReturn);
   } catch (Exception e) {
      System.err.println(
        "Error: Policy caused an exception" +
        " - execution halted");
      mplExit.shutdown();
   }
   }

// START RULE NO. 2
   pointcut rule2(ObjectOutputStream mplTarget, Object msg):
      target(mplTarget) &&
      args(msg) &&
      call(void writeObject(Object));

   before(ObjectOutputStream mplTarget, Object msg):
      rule2(mplTarget, msg)
   {
   try {
      len = ((String)msg).length();
      if ( (len<lowerCharLimit) )
      {
         msgCounters[0] = msgCounters[0]+1;
      }
      if ( (len>=lowerCharLimit&&
        len<=upperCharLimit) )
      {
         msgCounters[1] = msgCounters[1]+1;
      }
      if ( (len>upperCharLimit) )
      {
         msgCounters[2] = msgCounters[2]+1;
      }
      msgCharCounter = msgCharCounter+len;
   } catch (Exception e) {
      System.err.println(
        "Error: Policy caused an exception" +
        " - execution halted");
      mplExit.shutdown();
   }
      try {
      if (!( msgCounters[0]<=allowedMsgCount[0]&&
        msgCounters[1]<=allowedMsgCount[1]&&
        msgCounters[2]<=allowedMsgCount[2]&&
        msgCharCounter<=allowedMsgCharCount ))
      { mplExit.shutdown(); }
      } catch (Exception e) {
         System.err.println(
           "Error: Policy caused an exception" +
           " - execution halted");
         mplExit.shutdown();
      }
   }

// START RULE NO. 3
   pointcut rule3(Socket mplTarget):
      target(mplTarget) &&
      call(OutputStream getOutputStream());

   before(Socket mplTarget):
      rule3(mplTarget)
   {
      try {
      if (!( sockets.contains(mplTarget) ))
      { mplExit.shutdown(); }
      } catch (Exception e) {
         System.err.println(
           "Error: Policy caused an exception" +
           " - execution halted");
         mplExit.shutdown();
      }
   }

   after(Socket mplTarget)
         returning(OutputStream mplReturn):
      rule3(mplTarget)
   {
   try {
      oStreams = oStreams.union(mplReturn);
   } catch (Exception e) {
      System.err.println(
        "Error: Policy caused an exception" +
        " - execution halted");
      mplExit.shutdown();
   }
   }

// START RULE NO. 4
   pointcut rule4():
      call(Socket.new(..));

   after() returning(Socket mplReturn):
      rule4()
   {
   try {
      sockets = sockets.union(mplReturn);
      oStreams = oStreams.union(mplReturn.
        getOutputStream());
   } catch (Exception e) {
      System.err.println(
        "Error: Policy caused an exception" +
        " - execution halted");
      mplExit.shutdown();
   }
   }
}