import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.rmi.Naming;
import java.rmi.Remote;
import java.util.ArrayList;
import java.util.regex.*;
import mpl.MPLSet;
import mpl.MPLShutdown;
import mpl.MPLSimpleShutdown;

// this aspect includes policies: 
// - policyset1
// - allowed_hosts_policy
// - socket_conn_policy
// - message_policy
public aspect policyset1 {
   private String[] allowedHosts$mplSetDummy = {
        "localhost", "127.0.0.1"};
   private MPLSet<String> allowedHosts = 
       new MPLSet<String>(allowedHosts$mplSetDummy);
   private Pattern allowedPatterns = Pattern.compile(
        "internal.*company");
   private String lastRMIHost;
   private Integer[] allowedMsgCount = {5, 6, 7};
   private int allowedMsgCharCount = 250;
   private Integer[] msgCounters = {0, 0, 0};
   private int msgCharCounter = 0;
   private int lowerCharLimit = 10;
   private int upperCharLimit = 30;
   private int len;
   private MPLSet<Socket> sockets =
       new MPLSet<Socket>();
   private MPLSet<OutputStream> oStreams =
       new MPLSet<OutputStream>();
   private MPLSet<ObjectOutputStream> ooStreams =
       new MPLSet<ObjectOutputStream>();

   // shutdown sequence
   private MPLShutdown mplExit =
      new MPLSimpleShutdown();

// START RULE NO. 1
   pointcut rule1(String url):
      args(url) &&
      call(static Remote Naming.lookup(String));

   before(String url):
      rule1(url)
   {
      try {
      if (!( url.split("/").length>2&&
        (allowedHosts.contains(url.split("/")[2])||
        allowedPatterns.matcher(url.split("/")[2]).matches()) ))
      { mplExit.shutdown(); }
      if (!( url.split("/").length>2 ))
      { mplExit.shutdown(); }
      } catch (Exception e) {
         System.err.println(
           "Error: Policy caused an exception" +
           " - execution halted");
         mplExit.shutdown();
      }
   }

   after(String url) returning(Remote mplReturn):
      rule1(url)
   {
   try {
      lastRMIHost = url.split("/")[2];
   } catch (Exception e) {
      System.err.println(
        "Error: Policy caused an exception" +
        " - execution halted");
      mplExit.shutdown();
   }
   }

// START RULE NO. 2
   pointcut rule2(OutputStream os):
      args(os) &&
      call(ObjectOutputStream.new(OutputStream));

   before(OutputStream os):
      rule2(os)
   {
      try {
      if (!( oStreams.contains(os) ))
      { mplExit.shutdown(); }
      } catch (Exception e) {
         System.err.println(
           "Error: Policy caused an exception" +
           " - execution halted");
         mplExit.shutdown();
      }
   }

   after(OutputStream os)
         returning(ObjectOutputStream mplReturn):
      rule2(os)
   {
   try {
      ooStreams = ooStreams.union(mplReturn);
   } catch (Exception e) {
      System.err.println(
        "Error: Policy caused an exception" +
        " - execution halted");
      mplExit.shutdown();
   }
   }

// START RULE NO. 3
   pointcut rule3(ObjectOutputStream mplTarget, Object msg):
      target(mplTarget) &&
      args(msg) &&
      call(void writeObject(Object));

   before(ObjectOutputStream mplTarget, Object msg):
      rule3(mplTarget, msg)
   {
   try {
      len = ((String)msg).length();
      if ( (len<lowerCharLimit) )
      {
         msgCounters[0] = msgCounters[0]+1;
      }
      if ( (len>=lowerCharLimit&&
        len<=upperCharLimit) )
      {
         msgCounters[1] = msgCounters[1]+1;
      }
      if ( (len>upperCharLimit) )
      {
         msgCounters[2] = msgCounters[2]+1;
      }
      msgCharCounter = msgCharCounter+len;
   } catch (Exception e) {
      System.err.println(
        "Error: Policy caused an exception" +
        " - execution halted");
      mplExit.shutdown();
   }
      try {
      if (!( msgCounters[0]<=allowedMsgCount[0]&&
        msgCounters[1]<=allowedMsgCount[1]&&
        msgCounters[2]<=allowedMsgCount[2]&&
        msgCharCounter<=allowedMsgCharCount ))
      { mplExit.shutdown(); }
      } catch (Exception e) {
         System.err.println(
           "Error: Policy caused an exception" +
           " - execution halted");
         mplExit.shutdown();
      }
   }

// START RULE NO. 4
   pointcut rule4(Socket mplTarget):
      target(mplTarget) &&
      call(OutputStream getOutputStream());

   before(Socket mplTarget):
      rule4(mplTarget)
   {
      try {
      if (!( sockets.contains(mplTarget) ))
      { mplExit.shutdown(); }
      } catch (Exception e) {
         System.err.println(
           "Error: Policy caused an exception" +
           " - execution halted");
         mplExit.shutdown();
      }
   }

   after(Socket mplTarget)
         returning(OutputStream mplReturn):
      rule4(mplTarget)
   {
   try {
      oStreams = oStreams.union(mplReturn);
   } catch (Exception e) {
      System.err.println(
        "Error: Policy caused an exception" +
        " - execution halted");
      mplExit.shutdown();
   }
   }

// START RULE NO. 5
   pointcut rule5(String url, int port):
      args(url, port) &&
      call(Socket.new(String, int));

   before(String url, int port):
      rule5(url, port)
   {
      try {
      if (!( allowedHosts.contains(url)||
        allowedPatterns.matcher(url).matches() ))
      { mplExit.shutdown(); }
      if (!( lastRMIHost!=null&&
        url.equals(lastRMIHost) ))
      { mplExit.shutdown(); }
      } catch (Exception e) {
         System.err.println(
           "Error: Policy caused an exception" +
           " - execution halted");
         mplExit.shutdown();
      }
   }

   after(String url, int port)
         returning(Socket mplReturn):
      rule5(url, port)
   {
   try {
      lastRMIHost = url;
   } catch (Exception e) {
      System.err.println(
        "Error: Policy caused an exception" +
        " - execution halted");
      mplExit.shutdown();
   }
   }

// START RULE NO. 6
   pointcut rule6():
      call(Socket.new(..));

   after() returning(Socket mplReturn):
      rule6()
   {
   try {
      sockets = sockets.union(mplReturn);
      oStreams = oStreams.union(mplReturn.
        getOutputStream());
   } catch (Exception e) {
      System.err.println(
        "Error: Policy caused an exception" +
        " - execution halted");
      mplExit.shutdown();
   }
   }
}